
<div>
<%-include('./layout.ejs', {
  body: include('./body.ejs'),
  footer: include('./footer.ejs')
})%>
</div>

<hr>

<div>
<%-include('./layout.ejs', {
  footer: include('./footer.ejs')
})%>
</div>
