<!DOCTYPE html>
<html lang="en">

<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>layout</title>
</head>

<body>
  <div>
    <h2><%=title%></h2>

    <% if (typeof body !== 'undefined') { %>
    <%- body %>
    <% } else { %>
    <p>This is the default body content.</p>
    <% } %>

    <div>description</div>

    <%- footer %>
  </div>

</body>

</html>
